# Useful functions for the analysis

# Function SilvsNclust : calculates the silhouette for any possible number of cluster.
# Silhouette will be used subsequently to select an 'optimal' number of clusters NC
SilvsNclust=function(d.st.mat,ncluster=2:(ncol(d.st.mat)-1),to.plot=F){
	print(ncluster)
	Pamado=lapply(ncluster,function(x)PAMfmado.R(x=d.st.mat,K=x))
	Silmado=lapply(Pamado,silhouette)
	AvgSil=sapply(Silmado,function(x)median(x[,3]))
	if(to.plot)
		plot(ncluster,AvgSil,type="l",ylab="Silhouette Score",xlab="Cluster number")
	list("Pamado"=Pamado,"Silmado"=Silmado,"db"=data.frame("nclust"=ncluster,"Sil"=AvgSil))
}


# Function associate.Cluster : from an existing clustering (argument post.pam), returns the cluster to which the station "num_post" belongs.
associate.Cluster <- function(num_post,post.pam){
	clustering=post.pam$clustering
	cluster.names=attr(clustering,"names")
	i.cluster=which(paste(num_post)==cluster.names) 
	clustering[i.cluster]
}


# Function generate_gbl : Implements the main clustering (for given number of cluster and representative type), and returns the time-series of th regional index (i.e. after aggregation steps).
# Inputs : 
#	d.st.mat : input data (matrix)
#	ncluster : nb of clusters (given)
#	representative : "mean" (default) or "medoid"
generate_gbl<- function(d.st.mat,ncluster,representative="mean"){
	# Clustering -- applied to d.st.mat
	PAM.gev=PAMfmado.R(x=d.st.mat,K=ncluster)
	# Computing silhouette
	silmado= silhouette(PAM.gev)
	pam.cluster=PAM.gev$clustering
	# For each cluster, compute the "representative"
	#	- mean (of stations within the cluster) or
	#	- medoid (the most representative individual series)
	if (representative == "medoid"){
	  if (!is.numeric(ncluster)) { stop("Error in generate_gbl: Geographical cluster with medoids!") }
	  d.rep = d.st.mat[,PAM.gev$medoid]
	  colnames(d.rep) = as.character(1:ncluster)
	}else{if(representative == "mean"){
	  d.rep = d.st.mat[,1:ncluster]+NA
	  colnames(d.rep) = as.character(1:ncluster)
	  for (i in 1:ncluster){
	    d.rep[,i] = apply(d.st.mat[,pam.cluster==i,drop=F],1,mean)
	  }	
	}}
	# All clusters are aggregated into one single time-series (mean)
	d.st.gbl = apply(d.rep,1,mean)
	# Computing Silhouette for all possible number of clusters
	SvsN=SilvsNclust(d.st.mat[,-1])
	# Output graph (3 panels)
	layout(matrix(c(1,2,3,3), 2, 2, byrow = TRUE))
	# Panel 1 : Silouhette
	nbm = which.min(SvsN$db[,2])
	par(plt=c(.25,.85,.25,.85),mgp=c(2.5,.7,0),cex.lab=1.2,cex.axis=1.2)
	plot(SvsN$db[1:nbm,],type="l",lwd=2,mar=c(4,4,1,1),xlab="",ylab="",xlim=c(0,50),xaxs="i")
	title(xlab="Number of clusters",ylab="Silhouette")
	axis(1,at=seq(0,50,10))
	mtext("a)",side=3,lwd=2,at=-15,line=2)
	# Panel 2 : maps of clusters
	par(plt=c(.1,.9,.1,.9))
	plot(ll$lon,ll$lat,type="p",xlim=c(0,1),ylim=c(0,1),xlab="",ylab="",mar=c(3,3,1,1),col=pal.qual(ncluster)[pam.cluster],pch=19,bg=pal.qual(ncluster)[pam.cluster])
	mtext("b)",side=3,lwd=2,at=1,line=.9)
	
	# Légende
	num.cluster = 1:ncluster
	#legend("top",bg="white",ncol=ncluster,legend=paste(num.cluster),col=pal.qual(ncluster)[num.cluster],pch=19)
	# Panel 3 : regional and single clusters time-series
	par(plt=c(.1,.95,.18,.9),cex.lab=1.2,cex.axis=1.2,las=1)
	plot(years,d.st.gbl,type="l",ylim=range(d.rep),yaxt="n",ylab="Scaled RX1d",xlab="Years")#,main=paste("Scaled Maximum of RR :", ncluster,"groups,", representative))
	axis(2,at=1:3,labels=T,tcl=-.6,mgp=c(4.5,1,0),font=1.2)
	axis(2,at=.5:4,labels=F,tcl=-.3)
	mtext("c)",side=3,lwd=2,at=1954.5,line=1)
	# Time-series for each cluster
	sapply(1:ncluster,function(x) lines( years,d.rep[,x],col=pal.qual(ncluster)[x]))
	# Regional time-series
	lines(years,d.st.gbl,col="black",lwd=2)
	# Legend
	#legend("top",bg="white",ncol=8,legend=c(paste(num.cluster),"mean"),col=c(pal.qual(ncluster)[num.cluster],"black"),pch=19)
	list(d.st.gbl,d.rep)
}



# Function Extract.lm : From a "running" (over time) linear regression, calculates the main indicator, i.e. the time evolution of the estimated change in precips (delta_precip), and the corresponding CI at 90% and 95%
# The linear regression is "running": made from a given starting year ('yref'), up to a moving ending year (typically 1965 to 2014); this allows us to  illustrate the emergence of the signal.
Extract.lm = function(lmo,rel=T){
# Note on the calculations
#   'delta' 	is the estimated change of the raw data, and corresponding stats.
#	As the algorithm is applied to normalised data, change is expressed as a fraction of the mean over the period 
#	e.g. "+30%" mean the value increases from 85% of the mean (t=y1) to 115% of the mean (t=yn).
#   'ref'	is the reference value which can be used as a reference, if one wants to express change relative to the first year of the TS. 
#	e.g. "+30% over the period" means "+30% of the estimated value in year y1", so (X(yn) - X(y1))/X(t1) = 1.3
#   'delta_rel' is the change relative to year 'yref'
#
# All vectors have the same structure:   [best-estimate, p-value, lower bound 90%, upper bound 90%, lower bound 95%, upper bound 95%, CC]

	# List of considered years
	years=unique(lmo$model$year)
	# Length of the period
	n=max(years)-min(years)
	s.lm=summary(lmo)
	yref=min(years)		# Reference year to compute relative change
	# Initialise vectors : change (:=delta), reference (:=ref), relative change (:=delta_rel)
	delta	= rep(NA,7)	# The change 7 columns: best-estimate, p-value, lower bound 90%, upper bound 90%, [idem 95%], CC
	names(delta) = c("best_est","p_value","low_bnd_90","up_bnd_90","low_bnd_95","up_bnd_95","CC")
	ref = delta		# The reference
	delta_rel = delta	# The relative change
	delta_rel[7] = cc_change*(n-1)/54	# WARNING : this number has to be revised according to the observed warming over the region of interest

	# The change
	# Best-estimate
	delta[1] = lmo$coef[2]*(n-1)
	ref[1] = lmo$fitted.values[paste(yref)]
	# p-value
	slmo = summary(lmo)
	delta[2] = slmo$coef[2,4]
	ref[2] = 1	# For compatibility
	# Confidence interval (90% then 95%)
	lmoc90 = confint(lmo,level=.9)
	delta[3:4] = lmoc90[2,]*(length(years)-1)
	ref[3:4] = mean(lmo$fitted.values) - lmoc90[2,]*(mean(years)-yref)
	lmoc95 = confint(lmo,level=.95)
	delta[5:6] = lmoc95[2,]*(length(years)-1)
	ref[5:6] = mean(lmo$fitted.values) - lmoc95[2,]*(mean(years)-yref)
	# CC : The Clausius-Clapeyron response
	delta[7] = delta_rel[7]*ref[1]
	ref[7] = ref[1]	# For compatibility

	# The relative change
	delta_rel = delta / ref		# Coordinate by coordinat division
	# Warning negative values
	delta_rel[ref<0] = 9.9e15	# If the (estimated) reference is negative, then the relative change is huge
	delta_rel[(ref+delta)<0] = -1	# If the (estimated) final value is negative, then the relative change is -100% = -1

  # Look which value to return
  if (rel){
	return(delta_rel)
  } else {
	return(delta)
  }
}




# Function make_plots: performs the intensity analysis for a given number of classes (ncluster)
# The following variable is just initialised to avoid any troubles
ncluster = 14
make_plots <- function(ncluster,representative="mean",plot=T,name="",rel=T){
	# INPUTS
	# ncluster: the number of clusters
	# representative: the type of representative for each cluster; can be "mean" or "medoid" (see Bernard, Naveau and Vrac, JClim, 2013 for more information)
	# plot: if "T", plots are saved (pdf)
	# name: suffix for the name of pdf files (graphs)
	# rel: 	if "T" calculations are made wrt to the starting year of the data
	#	if "F" calculations are made on the raw time-series, ie wrt the average over the period

	# OUTPUTS
	# Res: a 3x1 matrix, providing the results of linear trend analysis over the entire period ["IC90_inf","best_est","IC90_sup"]
	# d.rep: a NYxNC matrix (NY: nb of years, NC: nb of clusters), providing the TS of each cluster (ie after the 1st aggregation step)


	# Clustering
	# Let's take care if ncluster=1 (ie simple averagin of all time-series)
	if (ncluster==1){
	  d.st.gbl = apply(d.st.mat,1,mean)
	  d.rep = d.st.gbl
      	  }
	# Other cases: ncluster>1
	else{
	  # Figure 1: maps of clusters, silhouette criterion, time series of each cluster
	  # Filename
	  nompdf=paste("figures/Clustering_NC",ncluster,"_",representative,name,"_Fig3.pdf",sep="")
	  if (plot==T) {pdf(nompdf)}
	  d.st.gbl.full = generate_gbl(d.st.mat,ncluster=ncluster,representative=representative)
	  d.st.gbl = d.st.gbl.full[[1]]
	  d.rep = d.st.gbl.full[[2]]
	  dev.off()  }

	# Figure 2: time series of the regional indicator (eg mean of annual max), estimation CI of Delta_RR, pvalue
	# Filename
	nompdf=paste("figures/RegionalTS_NC",ncluster,"_",representative,name,"_Fig4.pdf",sep="")
	if (plot==T) { pdf(nompdf) }
	# The ending year for the moving trend analysis (hte starting year is always the first year available)
	year.fin=years[4:ny]
	D = as.data.frame(cbind(as.numeric(names(d.st.gbl)),d.st.gbl))
	colnames(D) = c("years","rr")
	# Simple linear regression on the univariate (regional) time-series
	fitByYearAvg=lapply(year.fin,function(x){lm(rr~years,data=subset(D,years <= x))})
	# Results
	rel_change = as.data.frame(t(sapply(fitByYearAvg,Extract.lm,rel=rel)))
	# Prepare the plot (3 panels)
	par(mfrow=c(3,1),mar=c(1,6,1,1),mgp=c(4.5,.8,0),las=1,cex.lab=1.4,cex.axis=1.2)
	par(fig=c(0,1,.7,1))
	# Panel 1 : time-series
	plot(years,d.st.gbl,xlab="Year", ylab="Scaled RX1d",font.lab=2,font=2,xaxt="n")
	axis(1,labels=F,tcl=-.6)
	axis(1,at=years,labels=F,tcl=-.3)
	abline(h=1,col="grey")
	lines(years,d.st.gbl)
	cc_trend = cc_change/ny*(years - mean(years))
	lines(years,1+cc_trend,col="green",lwd=1.5,lty=5)
	lmf=lm(d.st.gbl~years)
	lines(years,lmf$fitted.values,col="black",lwd=1.5,lty=2)
	# Panel 2 : Delta_RR and corresponding CI
	# The range of values to be plotted
	ylim = with(rel_change[year.fin>years[20],],range(best_est,low_bnd_95,up_bnd_95,CC))
	# The plot itself, step by step
	par(fig=c(0,1,.4,.7),new=T)
	with(rel_change,plot(year.fin,best_est,xlim=range(years),ylim=ylim,xlab="1961 - XXXX",xaxt="n",yaxt="n",new=F,ylab="Relative change",font.lab=2,font=2))
	axis(2,at=c(-.1,0,.1,.2,.3),labels=c("-10%","0%","+10%","+20%","+30%"),las=1,font=2)
	axis(1,labels=F,tcl=-.6)
	axis(1,at=years,labels=F,tcl=-.3)
	with(rel_change,lines(year.fin,best_est,ylim=ylim))
	with(rel_change,lines(year.fin,low_bnd_95,col="red",lty=2))
	with(rel_change,lines(year.fin,up_bnd_95,col="red",lty=2))
	with(rel_change,lines(year.fin,low_bnd_90,col="blue",lty=2))
	with(rel_change,lines(year.fin,up_bnd_90,col="blue",lty=2))
	with(rel_change,lines(year.fin,CC,col="green",lwd=1.5,lty=5))
	abline(h=0,col="black",lty=2)
	# Panel 3 : P-value
	par(fig=c(0,1,0,.4),new=T,mar=c(7,6,1,1))
	with(rel_change,plot(year.fin,p_value,xlim=range(years),ylim=c(0,1),xlab="",xaxt="n",ylab="P-value",font.lab=2,font=2))
	with(rel_change,lines(year.fin,p_value,xlim=range(years),ylim=c(0,1),xlab="",xaxt="n",font.lab=2,font=2))
	axis(1,labels=T,tcl=-.6,mgp=c(4.5,1,0),font=2)
	axis(1,at=years,labels=F,tcl=-.3)
	title(xlab="Year",font.lab=2,mgp=c(3,1,0))
	abline(h=0.05,col="red",lty=2)
	abline(h=0.10,col="blue",lty=2)
	dev.off()

	# The returned objects
	nyf = length(year.fin)
	res.tmp = c(rel_change$low_bnd_90[nyf],rel_change$best_est[nyf],rel_change$up_bnd_90[nyf])
	Res = as.matrix(res.tmp)
	rownames(Res) = c("IC90_inf","best_est","IC90_sup")
	list(Res,d.rep)
}








